import { __awaiter } from 'tslib';
import { InjectionToken, ɵɵdefineInjectable, ɵɵinject, Injectable, Inject, NgModule } from '@angular/core';
import { HttpParams, HttpClient, HttpClientModule } from '@angular/common/http';
import { switchMap, catchError, mergeMap, distinctUntilChanged, map, tap, filter } from 'rxjs/operators';
import { of, throwError, from, BehaviorSubject } from 'rxjs';
import { unflatter } from '@monabbous/unflatter';
import { isEqual } from 'lodash';
import { digger } from '@monabbous/object-digger';

const API_CONFIG = new InjectionToken('APIConfig');
// @dynamic
class NgApiWrapperService {
    constructor(http, apiConfig) {
        this.http = http;
        this.apiConfig = apiConfig;
        console.log(apiConfig);
    }
    static jsonToFormData(object) {
        return __awaiter(this, void 0, void 0, function* () {
            if (object instanceof FormData) {
                return object;
            }
            const formData = new FormData();
            let worthConverting = false;
            const append = (data, keys = []) => __awaiter(this, void 0, void 0, function* () {
                return yield new Promise((res) => __awaiter(this, void 0, void 0, function* () {
                    for (const key of Object.keys(data)) {
                        if (key === 'isFormArray') {
                            delete data[key];
                            continue;
                        }
                        const item = data[key];
                        const itemKey = [...keys, key].map((k, i) => i > 0 ? `[${k}]` : k).join('');
                        if (item instanceof Blob) {
                            formData.append(itemKey, item);
                            worthConverting = true;
                        }
                        else if ((Array.isArray(item) || (typeof item === 'object')) && item !== null && item !== undefined) {
                            yield append(item, [...keys, key]);
                        }
                        else {
                            formData.append(itemKey, item);
                        }
                    }
                    res(formData);
                }));
            });
            yield append(object);
            return worthConverting ? formData : object;
        });
    }
    getFullUrl(server, version) {
        var _a, _b;
        return ((_a = this.apiConfig.servers[server]) === null || _a === void 0 ? void 0 : _a.baseUrl) + ((_b = this.apiConfig.servers[server]) === null || _b === void 0 ? void 0 : _b.versions[version]);
    }
    handleServer(server) {
        if (!Object.keys(this.apiConfig.servers).includes(server)) {
            console.warn(`Ng Api Wrapper: Server '${server}' is not in the configuration, will use the defaultServer`);
            server = this.apiConfig.defaultServer;
        }
        return server;
    }
    handleServerVersion(server, version) {
        var _a;
        if (!Object.keys(this.apiConfig.servers[server].versions).includes(version)) {
            console.warn(`Ng Api Wrapper: Server '${server}' Api version '${version}' is not in the configuration, will use the defaultVersion`);
            version = (_a = this.apiConfig.servers[server]) === null || _a === void 0 ? void 0 : _a.defaultVersion;
        }
        return version;
    }
    handleResponse(request, parameters) {
        return request
            .pipe(switchMap(response => this.apiConfig.onSuccess ? this.apiConfig.onSuccess(response, parameters) : of(response)), catchError(response => this.apiConfig.onError ? this.apiConfig.onError(response, parameters) : throwError(response)));
    }
    get(request) {
        var _a, _b, _c;
        const next = (req) => {
            var _a;
            let { body = {}, version, server, } = req;
            const { path, outsource, options = {}, } = req;
            server = this.handleServer(server);
            version = this.handleServerVersion(server, version);
            body = unflatter(body);
            const params = Object.assign({}, body);
            delete params._method;
            options.params = body ? new HttpParams({ fromObject: params }) : options.params;
            // @ts-ignore
            let method = (...a) => this.http.get(...a);
            let args = [(outsource ? '' : this.getFullUrl(server, version)) + path, options];
            if ((_a = this.apiConfig) === null || _a === void 0 ? void 0 : _a.methodOverride) {
                // @ts-ignore
                method = (...a) => this.http.post(...a);
                args = [(outsource ? '' : this.getFullUrl(server, version)) + path, Object.assign({ _method: 'GET' }, body), options];
            }
            return from(NgApiWrapperService.jsonToFormData(body))
                .pipe(mergeMap(b => {
                body = b;
                // @ts-ignore
                return this.handleResponse(method(...args), {
                    baseUrl: this.getFullUrl(server, version),
                    path,
                    outsource,
                    body,
                    server,
                    version,
                    options,
                    method: 'GET'
                });
            }));
        };
        return (_c = (_b = (_a = this.apiConfig) === null || _a === void 0 ? void 0 : _a.intercept) === null || _b === void 0 ? void 0 : _b.call(_a, request, next)) !== null && _c !== void 0 ? _c : next(request);
    }
    post(request) {
        var _a, _b, _c;
        const next = (req) => {
            let { body = {}, version, server, } = req;
            const { path, outsource, options = {}, } = req;
            server = this.handleServer(server);
            version = this.handleServerVersion(server, version);
            body = unflatter(body);
            return from(NgApiWrapperService.jsonToFormData(body))
                .pipe(mergeMap(b => {
                body = b;
                // @ts-ignore
                return this.handleResponse(this.http.post(
                // @ts-ignore
                (outsource ? '' : this.getFullUrl(server, version)) + path, body, options), {
                    baseUrl: this.getFullUrl(server, version),
                    path,
                    outsource,
                    body,
                    server,
                    version,
                    options,
                    method: 'POST'
                });
            }));
        };
        return (_c = (_b = (_a = this.apiConfig) === null || _a === void 0 ? void 0 : _a.intercept) === null || _b === void 0 ? void 0 : _b.call(_a, request, next)) !== null && _c !== void 0 ? _c : next(request);
    }
    patch(request) {
        var _a, _b, _c;
        const next = (req) => {
            var _a;
            let { body = {}, version, server, } = req;
            const { path, outsource, options = {}, } = req;
            server = this.handleServer(server);
            version = this.handleServerVersion(server, version);
            body = unflatter(body);
            // @ts-ignore
            let method = (...a) => this.http.patch(...a);
            let args = [(outsource ? '' : this.getFullUrl(server, version)) + path, body, options];
            if ((_a = this.apiConfig) === null || _a === void 0 ? void 0 : _a.methodOverride) {
                // @ts-ignore
                method = (...a) => this.http.post(...a);
                args = [(outsource ? '' : this.getFullUrl(server, version)) + path, Object.assign({ _method: 'PATCH' }, body), options];
            }
            return from(NgApiWrapperService.jsonToFormData(body))
                .pipe(mergeMap(b => {
                body = b;
                // @ts-ignore
                return this.handleResponse(method(...args), {
                    baseUrl: this.getFullUrl(server, version),
                    path,
                    outsource,
                    body,
                    server,
                    version,
                    options,
                    method: 'PATCH'
                });
            }));
        };
        return (_c = (_b = (_a = this.apiConfig) === null || _a === void 0 ? void 0 : _a.intercept) === null || _b === void 0 ? void 0 : _b.call(_a, request, next)) !== null && _c !== void 0 ? _c : next(request);
    }
    put(request) {
        var _a, _b, _c;
        const next = (req) => {
            var _a;
            let { body = {}, version, server, } = req;
            const { path, outsource, options = {}, } = req;
            server = this.handleServer(server);
            version = this.handleServerVersion(server, version);
            body = unflatter(body);
            // @ts-ignore
            let method = (...a) => this.http.put(...a);
            let args = [(outsource ? '' : this.getFullUrl(server, version)) + path, body, options];
            if ((_a = this.apiConfig) === null || _a === void 0 ? void 0 : _a.methodOverride) {
                // @ts-ignore
                method = (...a) => this.http.post(...a);
                args = [(outsource ? '' : this.getFullUrl(server, version)) + path, Object.assign({ _method: 'PUT' }, body), options];
            }
            return from(NgApiWrapperService.jsonToFormData(body))
                .pipe(mergeMap(b => {
                body = b;
                // @ts-ignore
                return this.handleResponse(method(...args), {
                    baseUrl: this.getFullUrl(server, version),
                    path,
                    outsource,
                    body,
                    server,
                    version,
                    options,
                    method: 'PUT'
                });
            }));
        };
        return (_c = (_b = (_a = this.apiConfig) === null || _a === void 0 ? void 0 : _a.intercept) === null || _b === void 0 ? void 0 : _b.call(_a, request, next)) !== null && _c !== void 0 ? _c : next(request);
    }
    delete(request) {
        var _a, _b, _c;
        const next = (req) => {
            var _a;
            let { body = {}, version, server, } = req;
            const { path, outsource, options = {}, } = req;
            server = this.handleServer(server);
            version = this.handleServerVersion(server, version);
            body = unflatter(body);
            // @ts-ignore
            let method = (...a) => this.http.delete(...a);
            let args = [(outsource ? '' : this.getFullUrl(server, version)) + path, body, options];
            if ((_a = this.apiConfig) === null || _a === void 0 ? void 0 : _a.methodOverride) {
                // @ts-ignore
                method = (...a) => this.http.post(...a);
                args = [(outsource ? '' : this.getFullUrl(server, version)) + path, Object.assign({ _method: 'DELETE' }, body), options];
            }
            return from(NgApiWrapperService.jsonToFormData(body))
                .pipe(mergeMap(b => {
                body = b;
                // @ts-ignore
                return this.handleResponse(method(...args), {
                    baseUrl: this.getFullUrl(server, version),
                    path,
                    outsource,
                    body,
                    server,
                    version,
                    options,
                    method: 'DELETE'
                });
            }));
        };
        return (_c = (_b = (_a = this.apiConfig) === null || _a === void 0 ? void 0 : _a.intercept) === null || _b === void 0 ? void 0 : _b.call(_a, request, next)) !== null && _c !== void 0 ? _c : next(request);
    }
}
NgApiWrapperService.ɵprov = ɵɵdefineInjectable({ factory: function NgApiWrapperService_Factory() { return new NgApiWrapperService(ɵɵinject(HttpClient), ɵɵinject(API_CONFIG)); }, token: NgApiWrapperService, providedIn: "root" });
NgApiWrapperService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
NgApiWrapperService.ctorParameters = () => [
    { type: HttpClient },
    { type: undefined, decorators: [{ type: Inject, args: [API_CONFIG,] }] }
];

// @dynamic
class NgApiWrapperModule {
    static forRoot(apiConfig) {
        return {
            ngModule: NgApiWrapperModule,
            providers: [
                { provide: API_CONFIG, useValue: apiConfig }
            ]
        };
    }
}
NgApiWrapperModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [HttpClientModule],
                exports: [],
                providers: [
                    NgApiWrapperService,
                ]
            },] }
];

function smartResourcePageRefresh(unique = 'id') {
    return distinctUntilChanged((pre, cur) => {
        var _a, _b, _c, _d;
        const preData = pre.data;
        const curData = cur.data;
        if (curData.length !== preData.length) {
            return false;
        }
        if (cur.meta && cur.meta && ((_a = cur.meta) === null || _a === void 0 ? void 0 : _a.pagination)
            && pre.meta && pre.meta && ((_b = pre.meta) === null || _b === void 0 ? void 0 : _b.pagination)
            && ((_c = pre.meta) === null || _c === void 0 ? void 0 : _c.pagination.current_page) !== ((_d = cur.meta) === null || _d === void 0 ? void 0 : _d.pagination.current_page)) {
            return false;
        }
        for (const datum of preData) {
            const newDatum = curData.find((d) => (digger(unique, d) || []).pop() === (digger(unique, datum) || []).pop());
            if (!newDatum) {
                return false;
            }
            if (!isEqual(datum, newDatum)) {
                Object.assign(datum, newDatum);
            }
        }
        return true;
    });
}
class NgApiResourceService {
    constructor(http) {
        this.http = http;
        this.resourceName = '';
        this.parentPrefix = '';
        this.params = {};
        this.filters = {};
        this.sort_order = 'asc';
        this.adapters = {};
        this.cachedModel$ = new BehaviorSubject(null);
        this.refresher$ = new BehaviorSubject(null);
        this.loadmore$ = new BehaviorSubject(null);
        this.lastPage = false;
    }
    getResourcePath(operation, body, id) {
        return ['get', 'create'].includes(operation) ?
            this.resourceName : `${this.resourceName}/${id}`;
    }
    getParentPrefix(operation, body) {
        return this.parentPrefix;
    }
    getDefaultServer(operation, body) {
        return this.defaultServer;
    }
    getDefaultVersion(operation, body) {
        return this.defaultVersion;
    }
    downAdapt(data) {
        for (const key of Object.keys(this.adapters)) {
            if (!this.adapters[key].down) {
                continue;
            }
            const keys = key.split(/\[\s*([\w]+)\s*]/g)
                .filter(f => f !== '');
            keys.reduce((pointer, k, i) => {
                if (![null, undefined].includes(pointer)) {
                    if (i < keys.length - 1) {
                        return pointer[k];
                    }
                    else {
                        pointer[k + '[adapted]'] = this.adapters[key].down(pointer[k], data);
                    }
                }
                return pointer;
            }, data);
        }
        return data;
    }
    upAdapt(data, method) {
        for (const key of Object.keys(this.adapters)) {
            if (!this.adapters[key].up) {
                continue;
            }
            const keys = key.split(/\[\s*([\w]+)\s*]/g)
                .filter(f => f !== '');
            keys.reduce((pointer, k, i) => {
                if (pointer !== undefined) {
                    if (i < keys.length - 1) {
                        return pointer[k];
                    }
                    else {
                        pointer[k] = this.adapters[key].up(pointer[k], data, method);
                    }
                }
                return pointer;
            }, data);
        }
        return data;
    }
    transformer(t) {
        return t;
    }
    where(field, value) {
        this.filters[field] = value;
        return this;
    }
    setParam(key, value) {
        this.params[key] = value;
        return this;
    }
    sortBy(field, order = 'asc') {
        this.sort_by = field;
        this.sort_order = order;
        return this;
    }
    get(filters = {}) {
        filters = Object.assign(Object.assign({}, this.filters), filters);
        filters = this.upAdapt(filters, 'get');
        let body = this.parseSort(this.parseFilters(this.params, filters, 'get'));
        return this.http.get({
            version: this.getDefaultVersion('get', body),
            server: this.getDefaultServer('get', body),
            path: this.getParentPrefix('get', body) + this.getResourcePath('get', body),
            body
        })
            .pipe(map((resource) => {
            if (!(Object.keys(resource).includes('data'))) {
                if (Object.keys(resource).includes(this.accessor)) {
                    // @ts-ignore
                    resource.data = resource[this.accessor];
                    delete resource[this.accessor];
                }
                else {
                    // @ts-ignore
                    resource.data = resource;
                }
            }
            if (!(Object.keys(resource).includes('meta')
                && Object.keys(resource.meta).includes('pagination'))
                && Object.keys(resource).includes('current_page')) {
                // @ts-ignore
                resource.meta = resource.meta || {};
            }
            resource.data = resource.data.map(d => this.downAdapt(d));
            resource.data = resource.data.map(d => this.transformer(d));
            return resource;
        }));
    }
    find(id) {
        const filters = this.upAdapt(this.filters, 'find');
        let body = this.parseFilters(this.params, filters, 'find');
        return this.http.get({
            version: this.getDefaultVersion('find', body),
            server: this.getDefaultServer('find', body),
            path: this.getParentPrefix('find', body) + this.getResourcePath('find', body, id),
            body
        })
            .pipe(map((resource) => {
            if (!('data' in resource)) {
                // @ts-ignore
                resource.data = resource;
            }
            resource.data = this.downAdapt(resource.data);
            resource.data = this.transformer(resource.data);
            return resource;
        }), tap(model => this.cachedModel$.next(model)));
    }
    create(body) {
        body = this.upAdapt(body, 'create');
        return this.http.post({
            version: this.getDefaultVersion('create', body),
            server: this.getDefaultServer('create', body),
            path: this.getParentPrefix('create', body) + this.getResourcePath('create', body),
            body
        });
    }
    update(id, body) {
        body = this.upAdapt(body, 'update');
        return this.http.patch({
            version: this.getDefaultVersion('update', body),
            server: this.getDefaultServer('update', body),
            path: this.getParentPrefix('update', body) + this.getResourcePath('update', body, id),
            body
        });
    }
    delete(id, body = {}) {
        body = this.upAdapt(body, 'delete');
        return this.http.delete({
            version: this.getDefaultVersion('delete', body),
            server: this.getDefaultServer('delete', body),
            path: this.getParentPrefix('delete', body) + this.getResourcePath('delete', body, id),
            body
        });
    }
    init(options) {
        let pagination = null;
        let reset = true;
        this.refresher$ = (options === null || options === void 0 ? void 0 : options.refresher$) || this.refresher$;
        const checkParent = () => {
            if (!(options === null || options === void 0 ? void 0 : options.parent)) {
                this.parentPrefix = '';
                return of(null);
            }
            return options.parent.cachedModel$
                .pipe(filter(parent => !!parent), tap(parent => this.parentPrefix = options.parent.resourceName + '/' + parent.data.id + '/'));
        };
        this.page$ =
            this.refresher$
                .pipe(switchMap(checkParent), switchMap(() => { var _a; return ((_a = options === null || options === void 0 ? void 0 : options.route) === null || _a === void 0 ? void 0 : _a.queryParams) || of({}); }), switchMap(filters => {
                if (options === null || options === void 0 ? void 0 : options.loadmore) {
                    reset = true;
                    this.lastPage = false;
                    pagination = null;
                    return this.loadmore$
                        .pipe(filter(() => !this.lastPage), map(() => (Object.assign(Object.assign({}, filters), { page: (pagination && pagination.current_page || 0) + 1 }))));
                }
                return of(filters);
            }), switchMap(filters => {
                var _a, _b;
                const newFilters = ((_a = options === null || options === void 0 ? void 0 : options.filters) === null || _a === void 0 ? void 0 : _a.length) ? {} : Object.assign({}, filters);
                for (const key of Object.keys(filters)) {
                    if ((_b = options === null || options === void 0 ? void 0 : options.filters) === null || _b === void 0 ? void 0 : _b.includes(key)) {
                        newFilters[key] = filters[key];
                    }
                }
                return this.get(newFilters);
            }), tap(page => { var _a; return pagination = (_a = page.meta) === null || _a === void 0 ? void 0 : _a.pagination; }), tap(page => {
                var _a, _b, _c, _d, _e, _f;
                return this.lastPage =
                    Math.ceil(((_b = (_a = page === null || page === void 0 ? void 0 : page.meta) === null || _a === void 0 ? void 0 : _a.pagination) === null || _b === void 0 ? void 0 : _b.total) / ((_d = (_c = page === null || page === void 0 ? void 0 : page.meta) === null || _c === void 0 ? void 0 : _c.pagination) === null || _d === void 0 ? void 0 : _d.per_page))
                        <= ((_f = (_e = page === null || page === void 0 ? void 0 : page.meta) === null || _e === void 0 ? void 0 : _e.pagination) === null || _f === void 0 ? void 0 : _f.current_page);
            }), distinctUntilChanged((pre, cur) => {
                var _a, _b, _c, _d, _e, _f, _g;
                if (options === null || options === void 0 ? void 0 : options.loadmore) {
                    if (reset) {
                        return false;
                    }
                    if (!((_a = cur === null || cur === void 0 ? void 0 : cur.meta) === null || _a === void 0 ? void 0 : _a.pagination)) {
                        this.lastPage = true;
                        return false;
                    }
                    if (Math.ceil(((_c = (_b = cur === null || cur === void 0 ? void 0 : cur.meta) === null || _b === void 0 ? void 0 : _b.pagination) === null || _c === void 0 ? void 0 : _c.total) / ((_e = (_d = cur === null || cur === void 0 ? void 0 : cur.meta) === null || _d === void 0 ? void 0 : _d.pagination) === null || _e === void 0 ? void 0 : _e.per_page)) >= ((_g = (_f = cur === null || cur === void 0 ? void 0 : cur.meta) === null || _f === void 0 ? void 0 : _f.pagination) === null || _g === void 0 ? void 0 : _g.current_page)) {
                        pre.data.push(...cur.data);
                        cur.data = pre.data;
                    }
                }
                return false;
            }), tap(() => reset = false), smartResourcePageRefresh(options === null || options === void 0 ? void 0 : options.uniqueId));
        this.model$ =
            this.refresher$
                .pipe(switchMap(checkParent), switchMap(() => { var _a; return ((_a = options === null || options === void 0 ? void 0 : options.route) === null || _a === void 0 ? void 0 : _a.params) || of({}); }), switchMap(params => this.find(params[options === null || options === void 0 ? void 0 : options.idParameter])), tap(model => this.cachedModel$.next(model)));
        return this;
    }
    superviseRefreshers(...services) {
        services.forEach(service => service.refresher$ = this.refresher$);
        return this;
    }
    parseFilters(params, filters, operation) {
        return Object.assign(Object.assign({}, params), filters);
    }
    parseSort(params) {
        params.sort_by = this.sort_by;
        params.sort_order = this.sort_order;
        return params;
    }
}
NgApiResourceService.ɵprov = ɵɵdefineInjectable({ factory: function NgApiResourceService_Factory() { return new NgApiResourceService(ɵɵinject(NgApiWrapperService)); }, token: NgApiResourceService, providedIn: "any" });
NgApiResourceService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'any'
            },] }
];
NgApiResourceService.ctorParameters = () => [
    { type: NgApiWrapperService }
];

/*
 * Public API Surface of ng-api-wrapper2
 */

/**
 * Generated bundle index. Do not edit.
 */

export { API_CONFIG, NgApiResourceService, NgApiWrapperModule, NgApiWrapperService, smartResourcePageRefresh };
//# sourceMappingURL=monabbous-ng-api-wrapper.js.map
