import { BehaviorSubject, MonoTypeOperatorFunction, Observable } from 'rxjs';
import { ActivatedRoute } from '@angular/router';
import { NgApiWrapperService } from './ng-api-wrapper.service';
export declare type ResourceModel = {
    id?: any;
    [key: number]: any;
    [key: string]: any;
};
declare type ForceString<T> = T & string;
declare type AdaptedModel<T> = {
    [key in (keyof T) as `${ForceString<key>}[adapted]`]?: T[key];
} & T;
declare type ModelFilters<T> = {
    [key: string]: any;
} & Partial<T>;
export declare type ResourcePagination = {
    per_page: number;
    current_page: number;
    total: number;
};
export declare type ResourceMeta<M> = {
    pagination?: ResourcePagination;
} & M;
export declare type ResourcePage<T, P = object, M = object> = {
    data: T[];
    meta?: ResourceMeta<M>;
} & P;
export declare type ResourceItem<T, P = object, M = object> = {
    data: T;
    meta?: ResourceMeta<M>;
} & P;
export declare type ResourceOperation = 'get' | 'find' | 'create' | 'update' | 'delete';
export declare function smartResourcePageRefresh<T extends ResourceModel = object, P = object, M = object>(unique?: string): MonoTypeOperatorFunction<ResourcePage<T, P, M>>;
export declare class NgApiResourceService<T extends ResourceModel = object, P = object, M = object> {
    protected http: NgApiWrapperService;
    resourceName: string;
    parentPrefix: string;
    defaultServer: any;
    defaultVersion: any;
    accessor: any;
    params: any;
    filters: ModelFilters<T>;
    sort_by: keyof T | string;
    sort_order: 'desc' | 'asc';
    adapters: {
        [key in keyof T]?: {
            up?: (value: any, body?: any, method?: ResourceOperation) => T[key];
            down?: (value: any, body: any) => T[key];
        };
    };
    page$: Observable<ResourcePage<AdaptedModel<T>, P, M>>;
    model$: Observable<ResourceItem<AdaptedModel<T>>>;
    cachedModel$: BehaviorSubject<ResourceItem<T, object, object>>;
    refresher$: BehaviorSubject<any>;
    loadmore$: BehaviorSubject<any>;
    lastPage: boolean;
    getResourcePath(operation: ResourceOperation, body: any, id?: any): string;
    getParentPrefix(operation: ResourceOperation, body: any): string;
    getDefaultServer(operation: ResourceOperation, body: any): string;
    getDefaultVersion(operation: ResourceOperation, body: any): string;
    downAdapt(data: any): AdaptedModel<T>;
    upAdapt(data: any, method: ResourceOperation): T;
    constructor(http: NgApiWrapperService);
    transformer(t: T): T;
    where(field: keyof T | string, value: any): this;
    setParam(key: string, value: any): this;
    sortBy(field: keyof T | string, order?: 'desc' | 'asc'): this;
    get(filters?: {}): Observable<ResourcePage<AdaptedModel<T>, P, M>>;
    find(id: any): Observable<ResourceItem<AdaptedModel<T>, P, M>>;
    create<R = any>(body: any): Observable<R>;
    update<R = any>(id: any, body: any): Observable<R>;
    delete<R = any>(id: any, body?: any): Observable<R>;
    init(options?: {
        route?: ActivatedRoute;
        refresher$?: BehaviorSubject<any>;
        filters?: string[];
        idParameter?: string;
        uniqueId?: string;
        loadmore?: boolean;
        parent?: NgApiResourceService<any>;
    }): this;
    superviseRefreshers(...services: NgApiResourceService<any>[]): this;
    protected parseFilters(params: any, filters: any, operation: ResourceOperation): any;
    protected parseSort(params: any): any;
}
export {};
