import { InjectionToken } from '@angular/core';
import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs';
export declare const API_CONFIG: InjectionToken<string>;
interface ServerVersions {
    [key: string]: string;
}
interface Server {
    baseUrl: string;
    versions: ServerVersions;
    defaultVersion: keyof ServerVersions;
}
interface Servers {
    [key: string]: Server;
}
export interface APIConfig {
    servers: Servers;
    defaultServer: keyof Servers;
    methodOverride?: boolean;
    intercept?(request: Request, next: (request: Request) => Observable<any>): Observable<any>;
    onSuccess?(response: any, request: Request): Observable<any>;
    onError?(response: any, request: Request): Observable<any>;
}
interface Request {
    baseUrl?: string;
    method?: 'GET' | 'POST' | 'PATCH' | 'PUT' | 'DELETE';
    path: string;
    outsource?: boolean;
    server?: number | string;
    version?: number | string;
    body?: {
        [key: string]: any;
    };
    options?: {
        headers?: HttpHeaders | {
            [header: string]: string | string[];
        };
        observe?: 'body';
        params?: HttpParams | {
            [param: string]: string | string[];
        };
        reportProgress?: boolean;
        responseType?: 'json' | 'blob' | 'text' | 'arraybuffer';
        withCredentials?: boolean;
    };
}
export declare class NgApiWrapperService {
    private http;
    apiConfig: APIConfig;
    static jsonToFormData(object: any): Promise<object | FormData>;
    constructor(http: HttpClient, apiConfig: APIConfig);
    getFullUrl(server: any, version: any): string;
    handleServer(server: any): string;
    handleServerVersion(server: any, version: any): string;
    protected handleResponse<T>(request: Observable<T>, parameters: Request): Observable<T>;
    get<T>(request: Request): Observable<T>;
    post<T>(request: Request): Observable<T>;
    patch<T>(request: Request): Observable<T>;
    put<T>(request: Request): Observable<T>;
    delete<T>(request: Request): Observable<T>;
}
export {};
